@@ -543,7 +543,10 @@ status slot."
         ;; Copy string.
         (copy-ub8-to-system-area octets 0 string-sap 0 size)
         ;; NULL-terminate it
-        (system-area-ub8-fill 0 string-sap size 4)
+        ;; (As it says up top: "assume 4 byte is enough for everyone.")
+        (let ((sap (sap+ string-sap size)))
+          (setf (sap-ref-8 sap 0) 0 (sap-ref-8 sap 1) 0
+                (sap-ref-8 sap 2) 0 (sap-ref-8 sap 3) 0))
         ;; Put the pointer in the vector.
         (setf (sap-ref-sap vec-sap vec-index-offset) string-sap)
         ;; Advance string-sap for the next string.
@@ -589,7 +592,8 @@ status slot."
   (envp (* c-string))
   (pty-name c-string)
   (channel (array int 2))
-  (dir c-string))
+  (dir c-string)
+  (preserve-fds (* int)))
 
 #-win32
 (define-alien-routine wait-for-exec
@@ -649,6 +653,11 @@ status slot."
       (loop for arg in args
             collect (coerce arg 'simple-string))))
 
+(defmacro coerce-or-copy (object type)
+  `(if (typep ,object ,type)
+       (copy-seq ,object)
+       (coerce ,object ,type)))
+
 ;;; FIXME: There shouldn't be two semiredundant versions of the
 ;;; documentation. Since this is a public extension function, the
 ;;; documentation should be in the doc string. So all information from
@@ -694,27 +703,34 @@ status slot."
 ;;; the fork worked, and NIL if it did not.
 (defun run-program (program args
                     &key
-                    (env nil env-p)
-                    (environment
-                     (when env-p
-                       (unix-environment-sbcl-from-cmucl env))
-                     environment-p)
-                    (wait t)
-                    search
-                    #-win32 pty
-                    input
-                    if-input-does-not-exist
-                    output
-                    (if-output-exists :error)
-                    (error :output)
-                    (if-error-exists :error)
-                    status-hook
-                    (external-format :default)
-                    directory
-                    #+win32 (escape-arguments t)
-                    #+win32 (window nil))
+                      (env nil env-p)
+                      (environment
+                       (when env-p
+                         (unix-environment-sbcl-from-cmucl env))
+                       environment-p)
+                      (wait t)
+                      search
+                      #-win32 pty
+                      input
+                      if-input-does-not-exist
+                      output
+                      (if-output-exists :error)
+                      (error :output)
+                      (if-error-exists :error)
+                      status-hook
+                      (external-format :default)
+                      directory
+                      preserve-fds
+                      #+win32 (escape-arguments t)
+                      #+win32 (window nil))
   "RUN-PROGRAM creates a new process specified by PROGRAM.
-ARGS are passed as the arguments to the program.
+ARGS is a list of strings to be passed literally to the new program.
+In POSIX environments, this list becomes the array supplied as the second
+parameter to the execv() or execvp() system call, each list element becoming
+one array element. The strings should not contain shell escaping, as there is
+no shell involvement. Further note that while conventionally the process
+receives its own pathname in argv[0], that is automatic, and the 0th string
+should not be present in ARGS.
 
 The program arguments and the environment are encoded using the
 default external format for streams.
@@ -800,6 +816,10 @@ Users Manual for details about the PROCESS structure.
       Specifies the directory in which the program should be run.
       NIL (the default) means the directory is unchanged.
 
+   :PRESERVE-FDS
+      A sequence of file descriptors which should remain open in the child
+      process.
+
    Windows specific options:
    :ESCAPE-ARGUMENTS (default T)
       Controls escaping of the arguments passed to CreateProcess.
@@ -887,16 +907,26 @@ Users Manual for details about the PROCESS structure.
                                   args
                                   stdin stdout stderr
                                   search environment-vec directory
-                                  window))
+                                  window
+                                  preserve-fds))
                                #-win32
-                               (with-args (args-vec args)
-                                 (with-system-mutex (*spawn-lock*)
-                                   (spawn progname args-vec
-                                          stdin stdout stderr
-                                          (if search 1 0)
-                                          environment-vec pty-name
-                                          channel
-                                          directory))))
+                               (let ((preserve-fds
+                                       (and preserve-fds
+                                            (sort (coerce-or-copy preserve-fds
+                                                                  '(simple-array (signed-byte #.(alien-size int)) (*)))
+                                                  #'<))))
+                                 (with-pinned-objects (preserve-fds)
+                                   (with-args (args-vec args)
+                                     (with-system-mutex (*spawn-lock*)
+                                       (spawn progname args-vec
+                                              stdin stdout stderr
+                                              (if search 1 0)
+                                              environment-vec pty-name
+                                              channel
+                                              directory
+                                              (if preserve-fds
+                                                  (vector-sap preserve-fds)
+                                                  (int-sap 0))))))))
                          (unless (minusp child)
                            #-win32
                            (setf child (wait-for-exec child channel))
@@ -1296,11 +1326,12 @@ Users Manual for details about the PROCESS structure.
   if not available."
   (or sb-sys::*software-version*
       (setf sb-sys::*software-version*
-            (string-trim '(#\newline)
-                         (%with-output-to-string (stream)
+            (possibly-base-stringize
+             (string-trim '(#\newline)
+                          (%with-output-to-string (stream)
                            (run-program "/bin/uname"
                                         ;; "-r" on haiku just prints "1"
                                         ;; but "-v" prints some detail.
                                         #+haiku '("-v")
                                         #-haiku '("-r")
-                                        :output stream))))))
+                                        :output stream)))))))
