@@ -55,7 +55,8 @@ Unused by SBCL itself: reserved for user and applications.")
 (define-load-time-global *init-hooks* nil
   "A list of function designators which are called in an unspecified
 order when a saved core image starts up, after the system itself has
-been initialized.
+been initialized, but before non-user threads such as the finalizer
+thread have been started.
 
 Unused by SBCL itself: reserved for user and applications.")
 
@@ -148,8 +149,8 @@ these hooks.")
     `(let* ((,size ,initial-size)
             (,string (make-array ,size :element-type ',element-type))
             (,pointer 0))
-       (declare (type (integer 0 ,sb-xc:array-dimension-limit) ,size)
-                (type (integer 0 ,(1- sb-xc:array-dimension-limit)) ,pointer)
+       (declare (type (integer 0 ,array-dimension-limit) ,size)
+                (type (integer 0 ,(1- array-dimension-limit)) ,pointer)
                 (type (simple-array ,element-type (*)) ,string))
        (flet ((push-char (char)
                 (declare (optimize (sb-c::insert-array-bounds-checks 0)))
@@ -184,11 +185,6 @@ unspecified."
   `(sb-thread:with-recursive-lock ((hash-table-lock ,hash-table))
      ,@body))
 
-(defmacro with-locked-system-table ((hash-table) &body body)
-  `(sb-thread::with-recursive-system-lock
-       ((hash-table-lock ,hash-table))
-     ,@body))
-
 (defmacro find-package-restarts ((package-designator &optional reader)
                                  &body body)
   (let ((package `(or ,(if reader
@@ -286,3 +282,20 @@ unspecified."
          `(defvar ,name ,@(when valuep (list value))))
         ((:final)
          `',name))))
+
+(define-load-time-global *deprecated-exports* nil)
+
+(defun deprecate-export (package symbol state version)
+  (declare (type deprecation-state state)
+           (type string version)
+           (type symbol symbol))
+  (setf (getf (getf *deprecated-exports* package) symbol)
+        (cons state version)))
+
+(defun check-deprecated-export (package symbol)
+  (let ((state (getf (getf *deprecated-exports* package) symbol)))
+    (when state
+      (deprecation-warn (car state) "SBCL" (cdr state) 'symbol
+                        (format nil "~A:~A" (package-name package) symbol)
+                        (list symbol))
+      t)))
