@@ -132,11 +132,11 @@ means to wait indefinitely.")
 ;;; cancelation points (system and C library calls) which check for cancelation,
 ;;; perform whatever cleanups were pushed by pthread_cleanup_push() and then stop.
 ;;;
-;;; P.S. To see that #+sb-thruption is no magic fix - suppose you have a stack
+;;; P.S. To see that #+sb-safepoint is no magic fix - suppose you have a stack
 ;;; with Lisp -> C -> -> Lisp where C acquired a resource and the currently
 ;;; top-of-stack Lisp function took a safepoint trap. It would be fine if all it
 ;;; wanted to do was GC, but is not safe in general.
-;;; Hence #+sb-thruption paints a dangerously attractive veneer over an unsafe
+;;; Hence #+sb-safepoint paints a dangerously attractive veneer over an unsafe
 ;;; concept, making it more subtly bad instead of very obviously bad.
 ;;;
 ;;; So this is a bad default. Bad bad bad. But it's backward-compatible.
@@ -157,7 +157,7 @@ means to wait indefinitely.")
                  (if *forcibly-terminate-threads-on-exit*
                      (sb-thread::%exit-other-threads)
                      (with-deadline (:seconds nil :override t)
-                       (sb-impl::finalizer-thread-stop)
+                       #+sb-thread (finalizer-thread-stop)
                        (sb-thread:grab-mutex sb-thread::*make-thread-lock*)))
                  (setf ok t))
             (os-exit code :abort (not ok)))))))
@@ -315,31 +315,28 @@ any non-negative real number."
 ;;; Flush anything waiting on one of the ANSI Common Lisp standard
 ;;; output streams before proceeding.
 (defun flush-standard-output-streams ()
-  (let ((null (make-broadcast-stream)))
-    (dolist (name '(*debug-io*
-                    *error-output*
-                    *query-io*
-                    *standard-output*
-                    *trace-output*
-                    *terminal-io*))
       ;; 0. Pull out the underlying stream, so we know what it is.
       ;; 1. Handle errors on it. We're doing this on entry to
       ;;    debugger, so we don't want recursive errors here.
       ;; 2. Rebind the stream symbol in case some poor sod sees
       ;;    a broken stream here while running with *BREAK-ON-ERRORS*.
-      (let ((stream (stream-output-stream (symbol-value name))))
-        ;; This is kind of crummy because it checks in globaldb for each
-        ;; stream symbol whether it can be bound to a stream. The translator
-        ;; for PROGV could skip ABOUT-TO-MODIFY-SYMBOL-VALUE based on
-        ;; an aspect of a policy, but if users figure that out they could
-        ;; do something horrible like rebind T and NIL.
-        (progv (list name) (list null)
-          (handler-bind ((stream-error
-                           (lambda (c)
-                             (when (eq stream (stream-error-stream c))
-                               (go :next)))))
-            (force-output stream))))
-      :next))
+  (flet ((flush (stream)
+           (let ((stream (stream-output-stream stream)))
+             (handler-bind ((stream-error
+                             (lambda (c)
+                               (when (eq (stream-error-stream c) stream)
+                                 (return-from flush)))))
+               (force-output stream)))))
+    (let ((null-stream (load-time-value *null-broadcast-stream* t)))
+      (macrolet ((flush-all (&rest streamvars)
+                   `(progn
+                      ,@(mapcar (lambda (streamvar)
+                                  `(let ((stream ,streamvar)
+                                         (,streamvar null-stream))
+                                     (flush stream)))
+                                streamvars))))
+        (flush-all *debug-io* *error-output* *query-io* *standard-output*
+                   *trace-output* *terminal-io*))))
   (values))
 
 (defun process-eval/load-options (options)
@@ -552,9 +549,9 @@ any non-negative real number."
                                                  (pathname it))
                                              :if-does-not-exist nil)
                        (cond (stream
-                              (dx-flet ((thunk ()
-                                          (load-as-source stream :context kind)))
-                                (sb-fasl::call-with-load-bindings #'thunk stream)))
+                              (sb-fasl::call-with-load-bindings
+                               (lambda (stream kind) (load-as-source stream :context kind))
+                               stream kind stream))
                              (specified-pathname
                               (cerror "Ignore missing init file"
                                       "The specified ~A file ~A was not found."
