@@ -62,10 +62,6 @@
   ;; Special forms which we don't currently handle, but might consider
   ;; supporting in the future are LOCALLY (with declarations),
   ;; MACROLET, SYMBOL-MACROLET and THE.
-  ;; Also, if (FLET ((F () ...)) (DEFUN A () ...) (DEFUN B () ...))
-  ;; were handled, then it would probably automatically work in
-  ;; the cold loader too, providing definitions for A and B before
-  ;; executing all other toplevel forms.
   (flet ((expand (form)
            (if expand
                (handler-case
@@ -188,30 +184,10 @@
             (let ((function (car form)))
               ;; Certain known functions have a special way of checking
               ;; their fopcompilability in the cross-compiler.
-              (or ;; allow %DEFUN, also ensuring that any inline lambda gets
-                  ;; its constant structures (possibly containing COMMAs) noted
-                  ;; as dumpable literals.
-                  (and (eq function 'sb-impl::%defun) (fopcompilable-p (fourth form)))
-                  (member function '(sb-pcl::!trivial-defmethod
-                                     sb-kernel::%defstruct))
-                  ;; allow DEF{CONSTANT,PARAMETER} only if the value form is ok
-                  (and (member function '(%defconstant sb-impl::%defparameter))
-                       (fopcompilable-p (third form)))
-                  (and (symbolp function) ; no ((lambda ...) ...)
-                       (get-properties (symbol-plist function)
-                                       '(:sb-cold-funcall-handler/for-effect
-                                         :sb-cold-funcall-handler/for-value)))
-                  (and (eq function 'setf)
-                       (fopcompilable-p (%macroexpand form *lexenv*)))
-                  (and (eq function 'sb-kernel:%svset)
-                       (destructuring-bind (thing index value) (cdr form)
-                         (and (symbolp thing)
-                              (integerp index)
-                              (eq (info :variable :kind thing) :global)
-                              (typep value
-                                     '(cons (member lambda function named-lambda))))))
-                  (and (eq function 'setq)
-                       (setq-fopcompilable-p (cdr form))))))))
+              (or (member function '(sb-pcl::!trivial-defmethod))
+                  ;; allow DEFCONSTANT only if the value form is ok
+                  (and (member function '(sb-impl::%defconstant))
+                       (fopcompilable-p (third form))))))))
 ) ; end FLET
 
 (defun let-fopcompilable-p (operator args)
@@ -256,13 +232,25 @@
 ;;; containing pointers; similarly (COMPLEX RATIONAL) and RATIO.
 (defun dumpable-leaflike-p (obj)
   (or (sb-xc:typep obj '(or symbol number character unboxed-array
-                            debug-name-marker
+                            system-area-pointer
                             #+sb-simd-pack simd-pack
                             #+sb-simd-pack-256 simd-pack-256))
+      (cl:typep obj 'debug-name-marker)
+      ;; STANDARD-OBJECT layouts use MAKE-LOAD-FORM, but all other layouts
+      ;; have the same status as symbols - composite objects but leaflike.
+      (and (typep obj 'wrapper) (not (layout-for-pcl-obj-p obj)))
+      ;; PACKAGEs are also leaflike.
+      (cl:typep obj 'package)
       ;; The cross-compiler wants to dump CTYPE instances as leaves,
       ;; but CLASSOIDs are excluded since they have a MAKE-LOAD-FORM method.
       #+sb-xc-host (cl:typep obj '(and ctype (not classoid)))
-      (sb-fasl:dumpable-layout-p obj)))
+      ;; FIXME: The target compiler wants to dump NAMED-TYPE instances,
+      ;; or maybe it doesn't, but we're forgetting to OPAQUELY-QUOTE them.
+      ;; For the moment I've worked around this with a backward-compatibility
+      ;; hack in FIND-CONSTANT which causes anonymous uses of #<named-type t>
+      ;; to be dumped as *UNIVERSAL-TYPE*.
+      ;; #+sb-xc (named-type-p obj)
+      ))
 
 ;;; Check that a literal form is fopcompilable. It would not be, for example,
 ;;; when the form contains structures with funny MAKE-LOAD-FORMS.
@@ -357,19 +345,20 @@
                        (fopcompile `(symbol-global-value ',form) path for-value-p))
                       (t
                        ;; Lexical
-                       (let* ((lambda-var (cdr (assoc form (lexenv-vars *lexenv*))))
-                              (handle (when lambda-var
-                                        (lambda-var-fop-value lambda-var))))
+                       (let* ((var (cdr (assoc form (lexenv-vars *lexenv*))))
+                              (handle (and (lambda-var-p var)
+                                           (leaf-info var))))
                          (cond (handle
-                                (setf (lambda-var-ever-used lambda-var) t)
+                                (setf (lambda-var-ever-used var) t)
                                 (when for-value-p
                                   (sb-fasl::dump-push handle fasl)))
                                (t
-                                ;; Undefined variable. Signal a warning, and
-                                ;; treat it as a special variable reference, like
-                                ;; the real compiler does -- do not elide even if
-                                ;; the value is unused.
-                                (note-undefined-reference form :variable)
+                                (unless var
+                                  ;; Undefined variable. Signal a warning, and
+                                  ;; treat it as a special variable reference, like
+                                  ;; the real compiler does -- do not elide even if
+                                  ;; the value is unused.
+                                  (note-undefined-reference form :variable))
                                 (fopcompile `(symbol-value ',form)
                                             path
                                             for-value-p))))))))))
@@ -417,7 +406,7 @@
                            (loop for (arg . next) on args
                              do (fopcompile arg path
                                             (if next nil for-value-p)))))
-                      ((setq #+sb-xc-host sb-fasl::setq-no-questions-asked)
+                      ((setq)
                        (if (and for-value-p (endp args))
                            (fopcompile nil path t)
                            (loop for (name value . next) on args by #'cddr
@@ -448,7 +437,7 @@
                                (let* ((obj (sb-fasl::dump-pop fasl))
                                       (var (make-lambda-var
                                             :%source-name name
-                                            :fop-value obj)))
+                                            :info obj)))
                                  (push var vars)
                                  (setf *lexenv*
                                        (make-lexenv
@@ -461,12 +450,13 @@
                                   (*compiler-error-context*
                                     (make-compiler-error-context
                                      :original-form form
-                                     :file-name (file-info-name file-info)
+                                     :file-name (file-info-truename file-info)
                                      :initialized t
                                      :file-position
                                      (nth-value 1 (find-source-root tlf *source-info*))
                                      :original-source-path (source-path-original-source path)
-                                     :lexenv *lexenv*)))
+                                     :handled-conditions
+                                     (lexenv-handled-conditions *lexenv*))))
                              (note-unreferenced-vars vars *policy*)))))
                       ;; Otherwise it must be an ordinary funcall.
                       (otherwise
