@@ -274,9 +274,9 @@
                   :constant-scs (single-sse-immediate)
                   :save-p t
                   :alternate-scs (single-sse-stack))
-  #+avx2
-  (avx2-reg float-registers
-   :locations #.*float-regs*)
+  (ymm-reg float-registers :locations #.*float-regs*)
+  ;; These next 3 should probably be named to YMM-{INT,SINGLE,DOUBLE}-REG
+  ;; but I think there are 3rd-party libraries that expect these names.
   #+sb-simd-pack-256
   (int-avx2-reg float-registers
                :locations #.*float-regs*
@@ -315,7 +315,7 @@
 (defparameter *oword-sc-names* '(sse-reg int-sse-reg single-sse-reg double-sse-reg
                                  int-sse-stack single-sse-stack double-sse-stack))
 #+sb-simd-pack-256
-(defparameter *hword-sc-names* '(avx2-reg int-avx2-reg single-avx2-reg double-avx2-reg
+(defparameter *hword-sc-names* '(ymm-reg int-avx2-reg single-avx2-reg double-avx2-reg
                                    int-avx2-stack single-avx2-stack double-avx2-stack))
 ) ; EVAL-WHEN
 (!define-storage-classes
@@ -424,9 +424,8 @@
 
                (static-symbol-p value))
        immediate-sc-number))
-    #+immobile-space
-    (layout
-       immediate-sc-number)
+    #+metaspace (sb-vm:layout (bug "Can't reference layout as a constant"))
+    #+(and immobile-space (not metaspace)) (wrapper immediate-sc-number)
     (single-float
        (if (eql value $0f0) fp-single-zero-sc-number fp-single-immediate-sc-number))
     (double-float
@@ -463,8 +462,8 @@
           (symbol   (if (static-symbol-p val)
                         (+ nil-value (static-symbol-offset val))
                         (make-fixup val :immobile-symbol)))
-          #+immobile-space
-          (layout
+          #+(and immobile-space (not metaspace))
+          (wrapper
            (make-fixup val :layout))
           (character (if tag
                          (logior (ash (char-code val) n-widetag-bits)
@@ -506,7 +505,16 @@
          (sb (sb-name (sc-sb sc)))
          (offset (tn-offset tn)))
     (ecase sb
-      (registers (reg-name (tn-reg tn)))
+      (registers
+       (concatenate 'string
+                    (reg-name (tn-reg tn))
+                    (case (sc-name (tn-sc tn))
+                      (descriptor-reg "(d)")
+                      (any-reg "(a)")
+                      (unsigned-reg "(u)")
+                      (signed-reg "(s)")
+                      (sap-reg "(p)")
+                      (t "(?)"))))
       (float-registers (format nil "FLOAT~D" offset))
       (stack (format nil "S~D" offset))
       (constant (format nil "Const~D" offset))
@@ -542,4 +550,7 @@
       (t
        (values :default nil)))))
 
-(defparameter *register-names* +qword-register-names+)
+(defvar *register-names* +qword-register-names+)
+
+(defmacro unbound-marker-bits ()
+  (logior (+ sb-vm:static-space-start #x100) unbound-marker-widetag))
