@@ -29,34 +29,15 @@
                    :disp (+ (* array-dimensions-offset n-word-bytes)
                             lowtag-mask)))
     (inst and bytes (lognot lowtag-mask))
-    (inst lea header (make-ea :dword :base rank
-                              :disp (fixnumize (1- array-dimensions-offset))))
-    (inst shl header n-widetag-bits)
+    ;; rank 1 is stored as 0, 2 is stored as 1, ...
+    (inst lea header (make-ea :dword :disp (fixnumize -1) :base rank))
+    (inst and header (fixnumize array-rank-mask))
+    (inst shl header array-rank-byte-pos)
     (inst or  header type)
-    (inst shr header 2)
+    (inst shr header n-fixnum-tag-bits)
     (pseudo-atomic ()
      (allocation nil bytes other-pointer-lowtag node nil result)
      (storew header result 0 other-pointer-lowtag))))
-
-(define-vop (make-array-header/c)
-  (:translate make-array-header)
-  (:policy :fast-safe)
-  (:arg-types (:constant t) (:constant t))
-  (:info type rank)
-   (:results (result :scs (descriptor-reg)))
-  (:node-var node)
-  (:generator 12
-    (let* ((header-size (+ rank
-                           (1- array-dimensions-offset)))
-           (bytes (logandc2 (+ (* (1+ header-size) n-word-bytes)
-                               lowtag-mask)
-                            lowtag-mask))
-           (header (logior (ash header-size
-                                n-widetag-bits)
-                           type)))
-     (pseudo-atomic ()
-      (allocation nil bytes other-pointer-lowtag node nil result)
-      (storew header result 0 other-pointer-lowtag)))))
 
 ;;;; additional accessors and setters for the array header
 (define-full-reffer %array-dimension *
@@ -67,16 +48,42 @@
   array-dimensions-offset other-pointer-lowtag
   (any-reg) positive-fixnum %set-array-dimension)
 
-(define-vop (array-rank-vop)
+(define-vop ()
   (:translate %array-rank)
   (:policy :fast-safe)
   (:args (x :scs (descriptor-reg)))
   (:results (res :scs (unsigned-reg)))
   (:result-types positive-fixnum)
   (:generator 6
-    (loadw res x 0 other-pointer-lowtag)
-    (inst shr res n-widetag-bits)
-    (inst sub res (1- array-dimensions-offset))))
+    (inst movzx res (make-ea :byte :disp (- 2 other-pointer-lowtag) :base x))
+    ;; not all registers have an addressable low byte,
+    ;; so the simple trick used on x86-64 won't work.
+    (inst inc res)
+    (inst and res array-rank-mask)))
+
+(define-vop ()
+  (:translate %array-rank=)
+  (:policy :fast-safe)
+  (:args (array :scs (descriptor-reg)))
+  (:info rank)
+  (:arg-types * (:constant t))
+  (:conditional :e)
+  (:generator 2
+    (inst cmp (make-ea :byte :disp (- 2 other-pointer-lowtag) :base array)
+          (encode-array-rank rank))))
+
+(define-vop (array-vectorp simple-type-predicate)
+  ;; SIMPLE-TYPE-PREDICATE says that it takes stack locations, but that's no good.
+  (:args (array :scs (any-reg descriptor-reg)))
+  (:translate vectorp)
+  (:conditional :z)
+  (:info)
+  (:guard (lambda (node)
+            (let ((arg (car (sb-c::combination-args node))))
+              (csubtypep (sb-c::lvar-type arg) (specifier-type 'array)))))
+  (:generator 1
+    (inst cmp (make-ea :byte :disp (- 2 other-pointer-lowtag) :base array)
+          (encode-array-rank 1))))
 
 ;;;; bounds checking routine
 (define-vop (check-bound)
@@ -158,17 +165,46 @@
 
 ;;;; accessors/setters
 
-;;; variants built on top of WORD-INDEX-REF, etc. I.e., those vectors
-;;; whose elements are represented in integer registers and are built
-;;; out of 8, 16, or 32 bit elements.
+;;; Ancestors
+(define-vop (dvref)
+  (:translate data-vector-ref-with-offset)
+  (:policy :fast-safe))
+(define-vop (dvset)
+  (:translate data-vector-set-with-offset)
+  (:policy :fast-safe))
+
+;;; variants which affect an entire lispword-sized value.
+(defmacro define-full-setter+addend (name type offset lowtag scs el-type)
+  `(progn
+     (define-vop (,name dvset)
+       (:policy :fast-safe)
+       (:args (object :scs (descriptor-reg))
+              (index :scs (any-reg immediate))
+              (value :scs ,scs))
+       (:info addend)
+       (:arg-types ,type tagged-num
+                   (:constant (constant-displacement ,lowtag n-word-bytes ,offset)) ,el-type)
+       (:generator 4
+         (sc-case index
+           (immediate
+            (inst mov (make-ea :dword :base object
+                               :disp (- (* (+ ,offset (tn-value index) addend)
+                                           n-word-bytes)
+                                        ,lowtag))
+                  value))
+           (t
+            (inst mov (make-ea :dword :base object :index index
+                               :disp (- (* (+ ,offset addend)
+                                           n-word-bytes) ,lowtag))
+                  value)))))))
 (macrolet ((def-full-data-vector-frobs (type element-type &rest scs)
              `(progn
-                (define-full-reffer+offset ,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" type)
+                (define-full-reffer+addend ,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" type)
                   ,type vector-data-offset other-pointer-lowtag ,scs
                   ,element-type data-vector-ref-with-offset)
-                (define-full-setter+offset ,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" type)
+                (define-full-setter+addend ,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" type)
                   ,type vector-data-offset other-pointer-lowtag ,scs
-                  ,element-type data-vector-set-with-offset))))
+                  ,element-type))))
   (def-full-data-vector-frobs simple-vector * descriptor-reg any-reg)
   (def-full-data-vector-frobs simple-array-unsigned-byte-32 unsigned-num
     unsigned-reg)
@@ -189,14 +225,12 @@
 ;;;; integer vectors whose elements are smaller than a byte, i.e.,
 ;;;; bit, 2-bit, and 4-bit vectors
 
-(define-vop (data-vector-ref-with-offset/simple-bit-vector-c)
-  (:translate data-vector-ref-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-ref-with-offset/simple-bit-vector-c dvref)
   (:args (object :scs (descriptor-reg)))
   (:arg-types simple-bit-vector
               (:constant (integer 0 #x7fffffff)) (:constant (integer 0 0)))
-  (:info index offset)
-  (:ignore offset)
+  (:info index addend)
+  (:ignore addend)
   (:results (result :scs (any-reg)))
   (:result-types positive-fixnum)
   (:generator 3
@@ -213,13 +247,11 @@
                (inst shl result (- right-shift))))))
     (inst and result (fixnumize 1))))
 
-(define-vop (data-vector-ref-with-offset/simple-bit-vector)
-  (:translate data-vector-ref-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-ref-with-offset/simple-bit-vector dvref)
   (:args (object :scs (descriptor-reg))
          (index :scs (unsigned-reg)))
-  (:info offset)
-  (:ignore offset)
+  (:info addend)
+  (:ignore addend)
   (:arg-types simple-bit-vector positive-fixnum (:constant (integer 0 0)))
   (:results (result :scs (any-reg)))
   (:result-types positive-fixnum)
@@ -236,19 +268,16 @@
                     (bit-shift (1- (integer-length elements-per-word))))
     `(progn
       ,@(unless (= bits 1)
-       `((define-vop (,(symbolicate 'data-vector-ref-with-offset/ type))
-         (:note "inline array access")
-         (:translate data-vector-ref-with-offset)
-         (:policy :fast-safe)
+       `((define-vop (,(symbolicate 'data-vector-ref-with-offset/ type) dvref)
          (:args (object :scs (descriptor-reg))
                 (index :scs (unsigned-reg)))
-         (:info offset)
+         (:info addend)
+         (:ignore addend)
          (:arg-types ,type positive-fixnum (:constant (integer 0 0)))
          (:results (result :scs (unsigned-reg) :from (:argument 0)))
          (:result-types positive-fixnum)
          (:temporary (:sc unsigned-reg :offset ecx-offset) ecx)
          (:generator 20
-           (aver (zerop offset))
            (move ecx index)
            (inst shr ecx ,bit-shift)
            (inst mov result (make-ea-for-vector-data object :index ecx))
@@ -263,16 +292,14 @@
                  (inst shl ecx ,(1- (integer-length bits)))))
            (inst shr result :cl)
            (inst and result ,(1- (ash 1 bits)))))
-       (define-vop (,(symbolicate 'data-vector-ref-with-offset/ type "-C"))
-         (:translate data-vector-ref-with-offset)
-         (:policy :fast-safe)
+       (define-vop (,(symbolicate 'data-vector-ref-with-offset/ type "-C") dvref)
          (:args (object :scs (descriptor-reg)))
          (:arg-types ,type (:constant index) (:constant (integer 0 0)))
-         (:info index offset)
+         (:info index addend)
+         (:ignore addend)
          (:results (result :scs (unsigned-reg)))
          (:result-types positive-fixnum)
          (:generator 15
-           (aver (zerop offset))
            (multiple-value-bind (word extra) (floor index ,elements-per-word)
              (loadw result object (+ word vector-data-offset)
                     other-pointer-lowtag)
@@ -280,23 +307,18 @@
                (inst shr result (* extra ,bits)))
              (unless (= extra ,(1- elements-per-word))
                (inst and result ,(1- (ash 1 bits)))))))))
-       (define-vop (,(symbolicate 'data-vector-set-with-offset/ type))
-         (:note "inline array store")
-         (:translate data-vector-set-with-offset)
-         (:policy :fast-safe)
+       (define-vop (,(symbolicate 'data-vector-set-with-offset/ type) dvset)
          (:args (object :scs (descriptor-reg) :to (:argument 2))
                 (index :scs (unsigned-reg) :target ecx)
-                (value :scs (unsigned-reg immediate) :target result))
-         (:info offset)
+                (value :scs (unsigned-reg immediate)))
+         (:info addend)
+         (:ignore addend)
          (:arg-types ,type positive-fixnum (:constant (integer 0 0))
                      positive-fixnum)
-         (:results (result :scs (unsigned-reg)))
-         (:result-types positive-fixnum)
          (:temporary (:sc unsigned-reg) word-index)
          (:temporary (:sc unsigned-reg) old)
          (:temporary (:sc unsigned-reg :offset ecx-offset :from (:argument 1)) ecx)
          (:generator 25
-           (aver (zerop offset))
            (move word-index index)
            (inst shr word-index ,bit-shift)
            (inst mov old (make-ea-for-vector-data object :index word-index))
@@ -321,25 +343,16 @@
               (inst or old value)))
            (inst rol old :cl)
            (inst mov (make-ea-for-vector-data object :index word-index)
-                 old)
-           (sc-case value
-             (immediate
-              (inst mov result (tn-value value)))
-             (unsigned-reg
-              (move result value)))))
-       (define-vop (,(symbolicate 'data-vector-set-with-offset/ type "-C"))
-         (:translate data-vector-set-with-offset)
-         (:policy :fast-safe)
+                 old)))
+       (define-vop (,(symbolicate 'data-vector-set-with-offset/ type "-C") dvset)
          (:args (object :scs (descriptor-reg))
-                (value :scs (unsigned-reg immediate) :target result))
+                (value :scs (unsigned-reg immediate)))
          (:arg-types ,type (:constant index) (:constant (integer 0 0))
                      positive-fixnum)
-         (:info index offset)
-         (:results (result :scs (unsigned-reg)))
-         (:result-types positive-fixnum)
+         (:info index addend)
+         (:ignore addend)
          (:temporary (:sc unsigned-reg :to (:result 0)) old)
          (:generator 20
-           (aver (zerop offset))
            (multiple-value-bind (word extra) (floor index ,elements-per-word)
              (loadw old object (+ word vector-data-offset) other-pointer-lowtag)
              (sc-case value
@@ -360,12 +373,7 @@
                   (inst or old value)
                   (unless (zerop shift)
                     (inst rol old shift)))))
-             (storew old object (+ word vector-data-offset) other-pointer-lowtag)
-             (sc-case value
-               (immediate
-                (inst mov result (tn-value value)))
-               (unsigned-reg
-                (move result value))))))))))
+             (storew old object (+ word vector-data-offset) other-pointer-lowtag))))))))
   (def-small-data-vector-frobs simple-bit-vector 1)
   (def-small-data-vector-frobs simple-array-unsigned-byte-2 2)
   (def-small-data-vector-frobs simple-array-unsigned-byte-4 4))
@@ -388,13 +396,10 @@
                           complex-offset)
                        other-pointer-lowtag)))))
 
-(define-vop (data-vector-ref-with-offset/simple-array-single-float)
-  (:note "inline array access")
-  (:translate data-vector-ref-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-ref-with-offset/simple-array-single-float dvref)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate)))
-  (:info offset)
+  (:info addend)
   (:arg-types simple-array-single-float tagged-num
               (:constant (constant-displacement other-pointer-lowtag
                                                 4 vector-data-offset)))
@@ -402,49 +407,31 @@
   (:result-types single-float)
   (:generator 5
    (with-empty-tn@fp-top(value)
-     (inst fld (float-ref-ea object index offset 4)))))
+     (inst fld (float-ref-ea object index addend 4)))))
 
-(define-vop (data-vector-set-with-offset/simple-array-single-float)
-  (:note "inline array store")
-  (:translate data-vector-set-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-set-with-offset/simple-array-single-float dvset)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (single-reg) :target result))
-  (:info offset)
+         (value :scs (single-reg)))
+  (:info addend)
   (:arg-types simple-array-single-float tagged-num
               (:constant (constant-displacement other-pointer-lowtag
                                                 4 vector-data-offset))
               single-float)
-  (:results (result :scs (single-reg)))
-  (:result-types single-float)
   (:generator 5
     (cond ((zerop (tn-offset value))
            ;; Value is in ST0.
-           (inst fst (float-ref-ea object index offset 4))
-           (unless (zerop (tn-offset result))
-             ;; Value is in ST0 but not result.
-             (inst fst result)))
+           (inst fst (float-ref-ea object index addend 4)))
           (t
            ;; Value is not in ST0.
            (inst fxch value)
-           (inst fst (float-ref-ea object index offset 4))
-           (cond ((zerop (tn-offset result))
-                  ;; The result is in ST0.
-                  (inst fst value))
-                 (t
-                  ;; Neither value or result are in ST0
-                  (unless (location= value result)
-                    (inst fst result))
-                  (inst fxch value)))))))
+           (inst fst (float-ref-ea object index addend 4))
+           (inst fxch value)))))
 
-(define-vop (data-vector-ref-with-offset/simple-array-double-float)
-  (:note "inline array access")
-  (:translate data-vector-ref-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-ref-with-offset/simple-array-double-float dvref)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate)))
-  (:info offset)
+  (:info addend)
   (:arg-types simple-array-double-float
               tagged-num
               (:constant (constant-displacement other-pointer-lowtag
@@ -453,51 +440,33 @@
   (:result-types double-float)
   (:generator 7
    (with-empty-tn@fp-top(value)
-     (inst fldd (float-ref-ea object index offset 8 :scale 2)))))
+     (inst fldd (float-ref-ea object index addend 8 :scale 2)))))
 
-(define-vop (data-vector-set-with-offset/simple-array-double-float)
-  (:note "inline array store")
-  (:translate data-vector-set-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-set-with-offset/simple-array-double-float dvset)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (double-reg) :target result))
-  (:info offset)
+         (value :scs (double-reg)))
+  (:info addend)
   (:arg-types simple-array-double-float tagged-num
               (:constant (constant-displacement other-pointer-lowtag
                                                 8 vector-data-offset))
               double-float)
-  (:results (result :scs (double-reg)))
-  (:result-types double-float)
   (:generator 20
     (cond ((zerop (tn-offset value))
            ;; Value is in ST0.
-           (inst fstd (float-ref-ea object index offset 8 :scale 2))
-           (unless (zerop (tn-offset result))
-                   ;; Value is in ST0 but not result.
-                   (inst fstd result)))
+           (inst fstd (float-ref-ea object index addend 8 :scale 2)))
           (t
            ;; Value is not in ST0.
            (inst fxch value)
-           (inst fstd (float-ref-ea object index offset 8 :scale 2))
-           (cond ((zerop (tn-offset result))
-                  ;; The result is in ST0.
-                  (inst fstd value))
-                 (t
-                  ;; Neither value or result are in ST0
-                  (unless (location= value result)
-                          (inst fstd result))
-                  (inst fxch value)))))))
+           (inst fstd (float-ref-ea object index addend 8 :scale 2))
+           (inst fxch value)))))
 
 ;;; complex float variants
 
-(define-vop (data-vector-ref-with-offset/simple-array-complex-single-float)
-  (:note "inline array access")
-  (:translate data-vector-ref-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-ref-with-offset/simple-array-complex-single-float dvref)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate)))
-  (:info offset)
+  (:info addend)
   (:arg-types simple-array-complex-single-float tagged-num
               (:constant (constant-displacement other-pointer-lowtag
                                                 8 vector-data-offset)))
@@ -506,64 +475,42 @@
   (:generator 5
     (let ((real-tn (complex-single-reg-real-tn value)))
       (with-empty-tn@fp-top (real-tn)
-        (inst fld (float-ref-ea object index offset 8 :scale 2))))
+        (inst fld (float-ref-ea object index addend 8 :scale 2))))
     (let ((imag-tn (complex-single-reg-imag-tn value)))
       (with-empty-tn@fp-top (imag-tn)
         ;; FIXME
-        (inst fld (float-ref-ea object index offset 8
+        (inst fld (float-ref-ea object index addend 8
                                          :scale 2 :complex-offset 4))))))
 
-(define-vop (data-vector-set-with-offset/simple-array-complex-single-float)
-  (:note "inline array store")
-  (:translate data-vector-set-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-set-with-offset/simple-array-complex-single-float dvset)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (complex-single-reg) :target result))
-  (:info offset)
+         (value :scs (complex-single-reg)))
+  (:info addend)
   (:arg-types simple-array-complex-single-float tagged-num
               (:constant (constant-displacement other-pointer-lowtag
                                                 8 vector-data-offset))
               complex-single-float)
-  (:results (result :scs (complex-single-reg)))
-  (:result-types complex-single-float)
   (:generator 5
-    (let ((value-real (complex-single-reg-real-tn value))
-          (result-real (complex-single-reg-real-tn result)))
+    (let ((value-real (complex-single-reg-real-tn value)))
       (cond ((zerop (tn-offset value-real))
              ;; Value is in ST0.
-             (inst fst (float-ref-ea object index offset 8 :scale 2))
-             (unless (zerop (tn-offset result-real))
-               ;; Value is in ST0 but not result.
-               (inst fst result-real)))
+             (inst fst (float-ref-ea object index addend 8 :scale 2)))
             (t
              ;; Value is not in ST0.
              (inst fxch value-real)
-             (inst fst (float-ref-ea object index offset 8 :scale 2))
-             (cond ((zerop (tn-offset result-real))
-                    ;; The result is in ST0.
-                    (inst fst value-real))
-                   (t
-                    ;; Neither value or result are in ST0
-                    (unless (location= value-real result-real)
-                      (inst fst result-real))
-                    (inst fxch value-real))))))
-    (let ((value-imag (complex-single-reg-imag-tn value))
-          (result-imag (complex-single-reg-imag-tn result)))
+             (inst fst (float-ref-ea object index addend 8 :scale 2))
+             (inst fxch value-real))))
+    (let ((value-imag (complex-single-reg-imag-tn value)))
       (inst fxch value-imag)
-      (inst fst (float-ref-ea object index offset 8
+      (inst fst (float-ref-ea object index addend 8
                                        :scale 2 :complex-offset 4))
-      (unless (location= value-imag result-imag)
-        (inst fst result-imag))
       (inst fxch value-imag))))
 
-(define-vop (data-vector-ref-with-offset/simple-array-complex-double-float)
-  (:note "inline array access")
-  (:translate data-vector-ref-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-ref-with-offset/simple-array-complex-double-float dvref)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate)))
-  (:info offset)
+  (:info addend)
   (:arg-types simple-array-complex-double-float tagged-num
               (:constant (constant-displacement other-pointer-lowtag
                                                 16 vector-data-offset)))
@@ -572,56 +519,37 @@
   (:generator 7
     (let ((real-tn (complex-double-reg-real-tn value)))
       (with-empty-tn@fp-top (real-tn)
-        (inst fldd (float-ref-ea object index offset 16 :scale 4)))
+        (inst fldd (float-ref-ea object index addend 16 :scale 4)))
     (let ((imag-tn (complex-double-reg-imag-tn value)))
       (with-empty-tn@fp-top (imag-tn)
-        (inst fldd (float-ref-ea object index offset 16
+        (inst fldd (float-ref-ea object index addend 16
                                           :scale 4 :complex-offset 8)))))))
 
-(define-vop (data-vector-set-with-offset/simple-array-complex-double-float)
-  (:note "inline array store")
-  (:translate data-vector-set-with-offset)
-  (:policy :fast-safe)
+(define-vop (data-vector-set-with-offset/simple-array-complex-double-float dvset)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (complex-double-reg) :target result))
-  (:info offset)
+         (value :scs (complex-double-reg)))
+  (:info addend)
   (:arg-types simple-array-complex-double-float tagged-num
               (:constant (constant-displacement other-pointer-lowtag
                                                 16 vector-data-offset))
               complex-double-float)
-  (:results (result :scs (complex-double-reg)))
-  (:result-types complex-double-float)
   (:generator 20
-    (let ((value-real (complex-double-reg-real-tn value))
-          (result-real (complex-double-reg-real-tn result)))
+    (let ((value-real (complex-double-reg-real-tn value)))
       (cond ((zerop (tn-offset value-real))
              ;; Value is in ST0.
-             (inst fstd (float-ref-ea object index offset 16
-                                               :scale 4))
-             (unless (zerop (tn-offset result-real))
-               ;; Value is in ST0 but not result.
-               (inst fstd result-real)))
+             (inst fstd (float-ref-ea object index addend 16
+                                               :scale 4)))
             (t
              ;; Value is not in ST0.
              (inst fxch value-real)
-             (inst fstd (float-ref-ea object index offset 16
+             (inst fstd (float-ref-ea object index addend 16
                                                :scale 4))
-             (cond ((zerop (tn-offset result-real))
-                    ;; The result is in ST0.
-                    (inst fstd value-real))
-                   (t
-                    ;; Neither value or result are in ST0
-                    (unless (location= value-real result-real)
-                      (inst fstd result-real))
-                    (inst fxch value-real))))))
-    (let ((value-imag (complex-double-reg-imag-tn value))
-          (result-imag (complex-double-reg-imag-tn result)))
+             (inst fxch value-real))))
+    (let ((value-imag (complex-double-reg-imag-tn value)))
       (inst fxch value-imag)
-      (inst fstd (float-ref-ea object index offset 16
+      (inst fstd (float-ref-ea object index addend 16
                                         :scale 4 :complex-offset 8))
-      (unless (location= value-imag result-imag)
-        (inst fstd result-imag))
       (inst fxch value-imag))))
 
 
@@ -630,12 +558,10 @@
 (macrolet ((define-data-vector-frobs (ptype element-type ref-inst
                                             8-bit-tns-p &rest scs)
   `(progn
-    (define-vop (,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" ptype))
-      (:translate data-vector-ref-with-offset)
-      (:policy :fast-safe)
+    (define-vop (,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" ptype) dvref)
       (:args (object :scs (descriptor-reg))
              (index :scs (signed-reg immediate)))
-      (:info offset)
+      (:info addend)
       (:arg-types ,ptype tagged-num
                   (:constant (constant-displacement other-pointer-lowtag
                                                     1 vector-data-offset)))
@@ -646,48 +572,41 @@
           (immediate
            (inst ,ref-inst value (make-ea-for-vector-data
                                   object :size :byte
-                                  :offset (+ (tn-value index) offset))))
+                                  :offset (+ (tn-value index) addend))))
           (t
            (inst ,ref-inst value
                  (make-ea-for-vector-data object :size :byte
-                                          :index index :offset offset))))))
-    (define-vop (,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" ptype))
-      (:translate data-vector-set-with-offset)
-      (:policy :fast-safe)
+                                          :index index :offset addend))))))
+    (define-vop (,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" ptype) dvset)
       (:args (object :scs (descriptor-reg) :to (:eval 0))
              (index :scs (signed-reg immediate) :to (:eval 0))
              (value :scs ,scs ,@(unless 8-bit-tns-p
                                   '(:target eax))))
-      (:info offset)
+      (:info addend)
       (:arg-types ,ptype tagged-num
                   (:constant (constant-displacement other-pointer-lowtag
                                                     1 vector-data-offset))
                   ,element-type)
       ,@(unless 8-bit-tns-p
-         '((:temporary (:sc unsigned-reg :offset eax-offset :target result
+         '((:temporary (:sc unsigned-reg :offset eax-offset
                         :from (:argument 2) :to (:result 0))
             eax)))
-      (:results (result :scs ,scs))
-      (:result-types ,element-type)
       (:generator 5
         ,@(unless 8-bit-tns-p
            '((move eax value)))
         (sc-case index
           (immediate
            (inst mov (make-ea-for-vector-data
-                      object :size :byte :offset (+ (tn-value index) offset))
+                      object :size :byte :offset (+ (tn-value index) addend))
                  ,(if 8-bit-tns-p
                       'value
                       'al-tn)))
           (t
            (inst mov (make-ea-for-vector-data object :size :byte
-                                              :index index :offset offset)
+                                              :index index :offset addend)
                  ,(if 8-bit-tns-p
                       'value
-                      'al-tn))))
-        (move result ,(if 8-bit-tns-p
-                          'value
-                          'eax)))))))
+                      'al-tn)))))))))
   (define-data-vector-frobs simple-array-unsigned-byte-7 positive-fixnum
     movzx nil unsigned-reg signed-reg)
   (define-data-vector-frobs simple-array-unsigned-byte-8 positive-fixnum
@@ -701,12 +620,10 @@
 ;;; {un,}signed-byte-16
 (macrolet ((define-data-vector-frobs (ptype element-type ref-inst &rest scs)
     `(progn
-      (define-vop (,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" ptype))
-        (:translate data-vector-ref-with-offset)
-        (:policy :fast-safe)
+      (define-vop (,(symbolicate "DATA-VECTOR-REF-WITH-OFFSET/" ptype) dvref)
         (:args (object :scs (descriptor-reg))
                (index :scs (signed-reg immediate)))
-        (:info offset)
+        (:info addend)
         (:arg-types ,ptype tagged-num
                     (:constant (constant-displacement other-pointer-lowtag
                                                       2 vector-data-offset)))
@@ -717,39 +634,34 @@
             (immediate
              (inst ,ref-inst value
                    (make-ea-for-vector-data object :size :word
-                                            :offset (+ (tn-value index) offset))))
+                                            :offset (+ (tn-value index) addend))))
             (t
              (inst ,ref-inst value
                    (make-ea-for-vector-data object :size :word
-                                            :index index :offset offset))))))
-      (define-vop (,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" ptype))
-        (:translate data-vector-set-with-offset)
-        (:policy :fast-safe)
+                                            :index index :offset addend))))))
+      (define-vop (,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/" ptype) dvset)
         (:args (object :scs (descriptor-reg) :to (:eval 0))
                (index :scs (signed-reg immediate) :to (:eval 0))
-               (value :scs ,scs :target eax))
-        (:info offset)
+               (value :scs ,scs))
+        (:info addend)
         (:arg-types ,ptype tagged-num
                     (:constant (constant-displacement other-pointer-lowtag
                                                       2 vector-data-offset))
                     ,element-type)
-        (:temporary (:sc unsigned-reg :offset eax-offset :target result
+        (:temporary (:sc unsigned-reg :offset eax-offset
                          :from (:argument 2) :to (:result 0))
                     eax)
-        (:results (result :scs ,scs))
-        (:result-types ,element-type)
         (:generator 5
           (move eax value)
           (sc-case index
             (immediate
              (inst mov (make-ea-for-vector-data
-                        object :size :word :offset (+ (tn-value index) offset))
+                        object :size :word :offset (+ (tn-value index) addend))
                    ax-tn))
             (t
              (inst mov (make-ea-for-vector-data object :size :word
-                                                :index index :offset offset)
-                   ax-tn)))
-          (move result eax))))))
+                                                :index index :offset addend)
+                   ax-tn))))))))
   (define-data-vector-frobs simple-array-unsigned-byte-15 positive-fixnum
     movzx unsigned-reg signed-reg)
   (define-data-vector-frobs simple-array-unsigned-byte-16 positive-fixnum
