@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; data object ref/set stuff
 
@@ -25,14 +25,27 @@
   (:args (object :scs (descriptor-reg))
          (value :scs (descriptor-reg any-reg immediate)))
   (:info name offset lowtag)
-  (:ignore name)
   (:results)
   (:generator 1
-     (storew (encode-value-if-immediate value) object offset lowtag)))
-
-(define-vop (init-slot set-slot)
-  (:info name dx-p offset lowtag)
-  (:ignore name dx-p))
+    (cond ((emit-code-page-write-barrier-p name)
+           (inst push (encode-value-if-immediate value))
+           (inst push offset)
+           (inst push object)
+           (when (= lowtag fun-pointer-lowtag)
+             (inst push eax-tn) ; spill eax to use as a temp
+             (loadw eax-tn object 0 fun-pointer-lowtag)
+             (inst shr eax-tn n-widetag-bits)
+             ;; increment index by number of boxed words
+             (inst add (make-ea :dword :base esp-tn :disp 8) eax-tn)
+             ;; and compute the code pointer from the fun pointer
+             (inst lea eax-tn
+                   (make-ea :dword :index eax-tn :scale n-word-bytes
+                            :disp (- fun-pointer-lowtag other-pointer-lowtag)))
+             (inst sub (make-ea :dword :base esp-tn :disp 4) eax-tn)
+             (inst pop eax-tn)) ; restore
+           (inst call (make-fixup 'code-header-set :assembly-routine)))
+          (t
+           (storew (encode-value-if-immediate value) object offset lowtag)))))
 
 (define-vop (compare-and-swap-slot)
   (:args (object :scs (descriptor-reg) :to :eval)
@@ -59,7 +72,7 @@
          (old :scs (descriptor-reg any-reg) :target eax)
          (new :scs (descriptor-reg any-reg)))
   (:temporary (:sc descriptor-reg :offset eax-offset) eax)
-  #!+sb-thread
+  #+sb-thread
   (:temporary (:sc descriptor-reg) tls)
   (:results (result :scs (descriptor-reg any-reg)))
   (:policy :fast-safe)
@@ -71,7 +84,7 @@
     (let ((unbound (generate-error-code vop 'unbound-symbol-error symbol))
           (check (gen-label)))
       (move eax old)
-      #!+sb-thread
+      #+sb-thread
       (progn
         (loadw tls symbol symbol-tls-index-slot other-pointer-lowtag)
         ;; Thread-local area, no LOCK needed.
@@ -110,7 +123,7 @@
       (inst cmp value unbound-marker-widetag)
       (inst jmp :e err-lab))))
 
-#!+sb-thread
+#+sb-thread
 (progn
   (define-vop (set)
     (:args (symbol :scs (descriptor-reg))
@@ -171,7 +184,7 @@
         (loadw value object symbol-value-slot other-pointer-lowtag)
         (emit-label ret-lab)))))
 
-#!-sb-thread
+#-sb-thread
 (progn
   (define-vop (symbol-value symbol-global-value)
     (:translate symeval))
@@ -179,7 +192,7 @@
     (:translate symeval))
   (define-vop (set %set-symbol-global-value)))
 
-#!+sb-thread
+#+sb-thread
 (define-vop (boundp)
   (:translate boundp)
   (:policy :fast-safe)
@@ -197,14 +210,14 @@
       (emit-label check-unbound-label)
       (inst cmp value unbound-marker-widetag))))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (boundp)
   (:translate boundp)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg)))
   (:conditional :ne)
   (:generator 9
-    (inst cmp (make-ea-for-object-slot object symbol-value-slot
+    (inst cmp (object-slot-ea object symbol-value-slot
                                        other-pointer-lowtag)
           unbound-marker-widetag)))
 
@@ -215,13 +228,15 @@
   (:args (symbol :scs (descriptor-reg)))
   (:results (res :scs (any-reg)))
   (:result-types positive-fixnum)
+  (:args-var args)
   (:generator 2
     ;; The symbol-hash slot of NIL holds NIL because it is also the
-    ;; cdr slot, so we have to strip off the two low bits to make sure
+    ;; car slot, so we have to strip off the two low bits to make sure
     ;; it is a fixnum.  The lowtag selection magic that is required to
     ;; ensure this is explained in the comment in objdef.lisp
     (loadw res symbol symbol-hash-slot other-pointer-lowtag)
-    (inst and res (lognot #b11))))
+    (unless (not-nil-tn-ref-p args)
+      (inst and res (lognot #b11)))))
 
 ;;;; fdefinition (FDEFN) objects
 
@@ -249,7 +264,7 @@
   (:temporary (:sc unsigned-reg) raw)
   (:results (result :scs (descriptor-reg)))
   (:generator 38
-    (inst lea raw (make-fixup 'closure-tramp :assembly-routine))
+    (inst mov raw (make-fixup 'closure-tramp :assembly-routine))
     (inst cmp (make-ea :byte :base function :disp (- fun-pointer-lowtag))
           simple-fun-widetag)
     (inst cmov :e raw
@@ -277,14 +292,14 @@
 ;;; symbol.
 ;;; See the "Chapter 9: Specials" of the SBCL Internals Manual.
 ;;
-#!+sb-thread
+#+sb-thread
 (macrolet
     ((def (vopname symbol-arg info sc-offset load-tls-index)
        `(define-vop (,vopname)
           (:args (val :scs (any-reg descriptor-reg)) ,@symbol-arg)
           ,@info
           (:temporary (:sc unsigned-reg ,@sc-offset) tls-index)
-          (:temporary (:sc unsigned-reg) bsp #!+win32 temp)
+          (:temporary (:sc unsigned-reg) bsp #+win32 temp)
           (:generator 10 ; cost is irrelevant. we explicitly pick a vop
            (load-binding-stack-pointer bsp)
            ,@load-tls-index
@@ -292,12 +307,12 @@
            (store-binding-stack-pointer bsp)
      ;; with-tls-ea on win32 causes tls-index to be an absolute address
      ;; which is problematic when UNBIND uses with-tls-ea too.
-           #!+win32(move temp tls-index)
+           #+win32(move temp tls-index)
            (with-tls-ea (EA :base tls-index :base-already-live-p t)
             (inst push EA :maybe-fs)
             (popw bsp (- binding-value-slot binding-size))
-            (storew #!-win32 tls-index
-                    #!+win32 temp
+            (storew #-win32 tls-index
+                    #+win32 temp
                     bsp (- binding-symbol-slot binding-size))
             (inst mov EA val :maybe-fs))))))
   (def bind ; bind a known symbol
@@ -312,7 +327,7 @@
         (inst call (make-fixup 'alloc-tls-index :assembly-routine))
         TLS-INDEX-VALID)))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (dynbind)
   (:args (val :scs (any-reg descriptor-reg))
          (symbol :scs (descriptor-reg)))
@@ -326,7 +341,7 @@
     (storew symbol bsp (- binding-symbol-slot binding-size))
     (storew val symbol symbol-value-slot other-pointer-lowtag)))
 
-#!+sb-thread
+#+sb-thread
 (define-vop (unbind)
   (:temporary (:sc unsigned-reg) temp bsp tls-index)
   (:generator 0
@@ -343,7 +358,7 @@
     (storew 0 bsp binding-value-slot)
     (store-binding-stack-pointer bsp)))
 
-#!-sb-thread
+#-sb-thread
 (define-vop (unbind)
   (:temporary (:sc unsigned-reg) symbol value bsp)
   (:generator 0
@@ -374,8 +389,8 @@
     (inst cmp symbol unbound-marker-widetag)
     (inst jmp :eq skip)
     (loadw value bsp binding-value-slot)
-    #!-sb-thread (storew value symbol symbol-value-slot other-pointer-lowtag)
-    #!+sb-thread (with-tls-ea (EA :base symbol :base-already-live-p t)
+    #-sb-thread (storew value symbol symbol-value-slot other-pointer-lowtag)
+    #+sb-thread (with-tls-ea (EA :base symbol :base-already-live-p t)
                    (inst mov EA value :maybe-fs))
     (storew 0 bsp binding-symbol-slot)
 
@@ -431,7 +446,7 @@
 
 ;;;; structure hackery
 
-(define-vop (instance-length)
+(define-vop ()
   (:policy :fast-safe)
   (:translate %instance-length)
   (:args (struct :scs (descriptor-reg)))
@@ -439,7 +454,7 @@
   (:result-types positive-fixnum)
   (:generator 4
     (loadw res struct 0 instance-pointer-lowtag)
-    (inst shr res n-widetag-bits)))
+    (inst shr res instance-length-shift)))
 
 (define-full-reffer instance-index-ref *
   instance-slots-offset instance-pointer-lowtag
@@ -464,22 +479,32 @@
 (define-full-reffer code-header-ref * 0 other-pointer-lowtag
   (any-reg descriptor-reg) * code-header-ref)
 
-(define-full-setter code-header-set * 0 other-pointer-lowtag
-  (any-reg descriptor-reg) * code-header-set)
+(define-vop (code-header-set)
+  (:translate code-header-set)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg))
+         (index :scs (unsigned-reg))
+         (value :scs (any-reg descriptor-reg)))
+  (:arg-types * unsigned-num *)
+  (:generator 10
+    (inst push value)
+    (inst push index)
+    (inst push object)
+    (inst call (make-fixup 'code-header-set :assembly-routine))))
 
 ;;;; raw instance slot accessors
 
-(defun make-ea-for-raw-slot (object index &optional (displacement 0))
+(defun instance-slot-ea (object index &optional (displacement 0))
   ;; instance-init vops pass a literal integer, ref/set can use an immediate tn
   (let ((imm-index (cond ((integerp index) index)
                          ((sc-is index immediate) (tn-value index)))))
     (make-ea :dword :base object
              ;; If index is a register, it needs no scaling - it has tag bits.
-             :index (unless imm-index index) :scale 1
+             :index (unless imm-index index)
              :disp (- (ash (+ (or imm-index 0) displacement instance-slots-offset)
                            word-shift) instance-pointer-lowtag))))
 
-(define-vop (raw-instance-ref/word)
+(define-vop ()
   (:translate %raw-instance-ref/word)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg)) (index :scs (any-reg immediate)))
@@ -487,30 +512,18 @@
   (:results (value :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 5
-    (inst mov value (make-ea-for-raw-slot object index))))
+    (inst mov value (instance-slot-ea object index))))
 
-(define-vop (raw-instance-set/word)
+(define-vop ()
   (:translate %raw-instance-set/word)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (unsigned-reg) :target result))
-  (:arg-types * tagged-num unsigned-num)
-  (:results (result :scs (unsigned-reg)))
-  (:result-types unsigned-num)
-  (:generator 5
-    (inst mov (make-ea-for-raw-slot object index) value)
-    (move result value)))
-
-(define-vop (raw-instance-init/word)
-  (:args (object :scs (descriptor-reg))
          (value :scs (unsigned-reg)))
-  (:arg-types * unsigned-num)
-  (:info index)
-  (:generator 5
-    (inst mov (make-ea-for-raw-slot object index) value)))
+  (:arg-types * tagged-num unsigned-num)
+  (:generator 5 (inst mov (instance-slot-ea object index) value)))
 
-(define-vop (raw-instance-ref/signed-word)
+(define-vop ()
   (:translate %raw-instance-ref/signed-word)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg)) (index :scs (any-reg immediate)))
@@ -518,28 +531,16 @@
   (:results (value :scs (signed-reg)))
   (:result-types signed-num)
   (:generator 5
-    (inst mov value (make-ea-for-raw-slot object index))))
+    (inst mov value (instance-slot-ea object index))))
 
-(define-vop (raw-instance-set/signed-word)
+(define-vop ()
   (:translate %raw-instance-set/signed-word)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (signed-reg) :target result))
-  (:arg-types * tagged-num signed-num)
-  (:results (result :scs (signed-reg)))
-  (:result-types signed-num)
-  (:generator 5
-    (inst mov (make-ea-for-raw-slot object index) value)
-    (move result value)))
-
-(define-vop (raw-instance-init/signed-word)
-  (:args (object :scs (descriptor-reg))
          (value :scs (signed-reg)))
-  (:arg-types * signed-num)
-  (:info index)
-  (:generator 5
-    (inst mov (make-ea-for-raw-slot object index) value)))
+  (:arg-types * tagged-num signed-num)
+  (:generator 5 (inst mov (instance-slot-ea object index) value)))
 
 (define-vop (raw-instance-atomic-incf/word)
   (:translate %raw-instance-atomic-incf/word)
@@ -551,10 +552,10 @@
   (:results (result :scs (unsigned-reg)))
   (:result-types unsigned-num)
   (:generator 5
-    (inst xadd (make-ea-for-raw-slot object index) diff :lock)
+    (inst xadd (instance-slot-ea object index) diff :lock)
     (move result diff)))
 
-(define-vop (raw-instance-ref/single)
+(define-vop ()
   (:translate %raw-instance-ref/single)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg)) (index :scs (any-reg immediate)))
@@ -563,42 +564,19 @@
   (:result-types single-float)
   (:generator 5
     (with-empty-tn@fp-top(value)
-      (inst fld (make-ea-for-raw-slot object index)))))
+      (inst fld (instance-slot-ea object index)))))
 
-(define-vop (raw-instance-set/single)
+(define-vop ()
   (:translate %raw-instance-set/single)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (single-reg) :target result))
-  (:arg-types * tagged-num single-float)
-  (:results (result :scs (single-reg)))
-  (:result-types single-float)
-  (:generator 5
-    (unless (zerop (tn-offset value))
-      (inst fxch value))
-    (inst fst (make-ea-for-raw-slot object index))
-    (cond
-      ((zerop (tn-offset value))
-        (unless (zerop (tn-offset result))
-          (inst fst result)))
-      ((zerop (tn-offset result))
-        (inst fst value))
-      (t
-        (unless (location= value result)
-          (inst fst result))
-        (inst fxch value)))))
-
-(define-vop (raw-instance-init/single)
-  (:args (object :scs (descriptor-reg))
          (value :scs (single-reg)))
-  (:arg-types * single-float)
-  (:info index)
+  (:arg-types * tagged-num single-float)
   (:generator 5
-    (with-tn@fp-top (value)
-      (inst fst (make-ea-for-raw-slot object index)))))
+    (with-tn@fp-top (value) (inst fst (instance-slot-ea object index)))))
 
-(define-vop (raw-instance-ref/double)
+(define-vop ()
   (:translate %raw-instance-ref/double)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg)) (index :scs (any-reg immediate)))
@@ -607,42 +585,19 @@
   (:result-types double-float)
   (:generator 5
     (with-empty-tn@fp-top(value)
-      (inst fldd (make-ea-for-raw-slot object index)))))
+      (inst fldd (instance-slot-ea object index)))))
 
-(define-vop (raw-instance-set/double)
+(define-vop ()
   (:translate %raw-instance-set/double)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (double-reg) :target result))
-  (:arg-types * tagged-num double-float)
-  (:results (result :scs (double-reg)))
-  (:result-types double-float)
-  (:generator 5
-    (unless (zerop (tn-offset value))
-      (inst fxch value))
-    (inst fstd (make-ea-for-raw-slot object index))
-    (cond
-      ((zerop (tn-offset value))
-        (unless (zerop (tn-offset result))
-          (inst fstd result)))
-      ((zerop (tn-offset result))
-        (inst fstd value))
-      (t
-        (unless (location= value result)
-          (inst fstd result))
-        (inst fxch value)))))
-
-(define-vop (raw-instance-init/double)
-  (:args (object :scs (descriptor-reg))
          (value :scs (double-reg)))
-  (:arg-types * double-float)
-  (:info index)
+  (:arg-types * tagged-num double-float)
   (:generator 5
-    (with-tn@fp-top (value)
-      (inst fstd (make-ea-for-raw-slot object index)))))
+    (with-tn@fp-top (value) (inst fstd (instance-slot-ea object index)))))
 
-(define-vop (raw-instance-ref/complex-single)
+(define-vop ()
   (:translate %raw-instance-ref/complex-single)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
@@ -653,63 +608,28 @@
   (:generator 5
     (let ((real-tn (complex-single-reg-real-tn value)))
       (with-empty-tn@fp-top (real-tn)
-        (inst fld (make-ea-for-raw-slot object index))))
+        (inst fld (instance-slot-ea object index))))
     (let ((imag-tn (complex-single-reg-imag-tn value)))
       (with-empty-tn@fp-top (imag-tn)
-        (inst fld (make-ea-for-raw-slot object index 1))))))
+        (inst fld (instance-slot-ea object index 1))))))
 
-(define-vop (raw-instance-set/complex-single)
+(define-vop ()
   (:translate %raw-instance-set/complex-single)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (complex-single-reg) :target result))
-  (:arg-types * positive-fixnum complex-single-float)
-  (:results (result :scs (complex-single-reg)))
-  (:result-types complex-single-float)
-  (:generator 5
-    (let ((value-real (complex-single-reg-real-tn value))
-          (result-real (complex-single-reg-real-tn result)))
-      (cond ((zerop (tn-offset value-real))
-             ;; Value is in ST0.
-             (inst fst (make-ea-for-raw-slot object index))
-             (unless (zerop (tn-offset result-real))
-               ;; Value is in ST0 but not result.
-               (inst fst result-real)))
-            (t
-             ;; Value is not in ST0.
-             (inst fxch value-real)
-             (inst fst (make-ea-for-raw-slot object index))
-             (cond ((zerop (tn-offset result-real))
-                    ;; The result is in ST0.
-                    (inst fst value-real))
-                   (t
-                    ;; Neither value or result are in ST0
-                    (unless (location= value-real result-real)
-                      (inst fst result-real))
-                    (inst fxch value-real))))))
-    (let ((value-imag (complex-single-reg-imag-tn value))
-          (result-imag (complex-single-reg-imag-tn result)))
-      (inst fxch value-imag)
-      (inst fst (make-ea-for-raw-slot object index 1))
-      (unless (location= value-imag result-imag)
-        (inst fst result-imag))
-      (inst fxch value-imag))))
-
-(define-vop (raw-instance-init/complex-single)
-  (:args (object :scs (descriptor-reg))
          (value :scs (complex-single-reg)))
-  (:arg-types * complex-single-float)
-  (:info index)
+  (:arg-types * positive-fixnum complex-single-float)
   (:generator 5
     (let ((value-real (complex-single-reg-real-tn value)))
       (with-tn@fp-top (value-real)
-        (inst fst (make-ea-for-raw-slot object index))))
+        (inst fst (instance-slot-ea object index))))
     (let ((value-imag (complex-single-reg-imag-tn value)))
-      (with-tn@fp-top (value-imag)
-        (inst fst (make-ea-for-raw-slot object index 1))))))
+      (inst fxch value-imag)
+      (inst fst (instance-slot-ea object index 1))
+      (inst fxch value-imag))))
 
-(define-vop (raw-instance-ref/complex-double)
+(define-vop ()
   (:translate %raw-instance-ref/complex-double)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
@@ -720,61 +640,26 @@
   (:generator 7
     (let ((real-tn (complex-double-reg-real-tn value)))
       (with-empty-tn@fp-top (real-tn)
-        (inst fldd (make-ea-for-raw-slot object index))))
+        (inst fldd (instance-slot-ea object index))))
     (let ((imag-tn (complex-double-reg-imag-tn value)))
       (with-empty-tn@fp-top (imag-tn)
-        (inst fldd (make-ea-for-raw-slot object index 2))))))
+        (inst fldd (instance-slot-ea object index 2))))))
 
-(define-vop (raw-instance-set/complex-double)
+(define-vop ()
   (:translate %raw-instance-set/complex-double)
   (:policy :fast-safe)
   (:args (object :scs (descriptor-reg))
          (index :scs (any-reg immediate))
-         (value :scs (complex-double-reg) :target result))
-  (:arg-types * positive-fixnum complex-double-float)
-  (:results (result :scs (complex-double-reg)))
-  (:result-types complex-double-float)
-  (:generator 20
-    (let ((value-real (complex-double-reg-real-tn value))
-          (result-real (complex-double-reg-real-tn result)))
-      (cond ((zerop (tn-offset value-real))
-             ;; Value is in ST0.
-             (inst fstd (make-ea-for-raw-slot object index))
-             (unless (zerop (tn-offset result-real))
-               ;; Value is in ST0 but not result.
-               (inst fstd result-real)))
-            (t
-             ;; Value is not in ST0.
-             (inst fxch value-real)
-             (inst fstd (make-ea-for-raw-slot object index))
-             (cond ((zerop (tn-offset result-real))
-                    ;; The result is in ST0.
-                    (inst fstd value-real))
-                   (t
-                    ;; Neither value or result are in ST0
-                    (unless (location= value-real result-real)
-                      (inst fstd result-real))
-                    (inst fxch value-real))))))
-    (let ((value-imag (complex-double-reg-imag-tn value))
-          (result-imag (complex-double-reg-imag-tn result)))
-      (inst fxch value-imag)
-      (inst fstd (make-ea-for-raw-slot object index 2))
-      (unless (location= value-imag result-imag)
-        (inst fstd result-imag))
-      (inst fxch value-imag))))
-
-(define-vop (raw-instance-init/complex-double)
-  (:args (object :scs (descriptor-reg))
          (value :scs (complex-double-reg)))
-  (:arg-types * complex-double-float)
-  (:info index)
+  (:arg-types * positive-fixnum complex-double-float)
   (:generator 20
     (let ((value-real (complex-double-reg-real-tn value)))
       (with-tn@fp-top (value-real)
-        (inst fstd (make-ea-for-raw-slot object index))))
+        (inst fstd (instance-slot-ea object index))))
     (let ((value-imag (complex-double-reg-imag-tn value)))
-      (with-tn@fp-top (value-imag)
-        (inst fstd (make-ea-for-raw-slot object index 2))))))
+      (inst fxch value-imag)
+      (inst fstd (instance-slot-ea object index 2))
+      (inst fxch value-imag))))
 
 ;;;;
 
