@@ -44,10 +44,6 @@
 #include "validate.h"
 #include "gc-internal.h"
 
-#if defined(LISP_FEATURE_SB_WTIMER) && !defined(LISP_FEATURE_DARWIN)
-# include <sys/event.h>
-#endif
-
 
 
 #ifdef __NetBSD__
@@ -122,12 +118,34 @@ os_context_sigmask_addr(os_context_t *context)
 }
 
 os_vm_address_t
-os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len)
+os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len, int executable, int jit)
 {
-    int protection = attributes & IS_GUARD_PAGE ? OS_VM_PROT_NONE : OS_VM_PROT_ALL;
-    attributes &= ~IS_GUARD_PAGE;
+    int protection;
     int flags = 0;
 
+    if (attributes & IS_GUARD_PAGE)
+        protection = OS_VM_PROT_NONE;
+    else
+#ifndef LISP_FEATURE_DARWIN_JIT
+        protection = OS_VM_PROT_ALL;
+#else
+    if (jit) {
+        if (jit == 2)
+            protection = OS_VM_PROT_ALL;
+        else
+            protection = OS_VM_PROT_READ | OS_VM_PROT_WRITE;
+        flags = MAP_JIT;
+    }
+    else if (executable) {
+        protection = OS_VM_PROT_READ | OS_VM_PROT_EXECUTE;
+    }
+    else {
+        protection = OS_VM_PROT_READ | OS_VM_PROT_WRITE;
+    }
+#endif
+
+    attributes &= ~IS_GUARD_PAGE;
+
 #ifndef LISP_FEATURE_DARWIN // Do not use MAP_FIXED, because the OS is sane.
 
     /* The *BSD family of OSes seem to ignore 'addr' when it is outside
@@ -216,7 +234,7 @@ os_validate(int attributes, os_vm_address_t addr, os_vm_size_t len)
                     " mounted with wxallowed?\n");
         else
 #endif
-        perror("mmap");
+            perror("mmap");
         return NULL;
     }
 
@@ -234,7 +252,11 @@ void
 os_protect(os_vm_address_t address, os_vm_size_t length, os_vm_prot_t prot)
 {
     if (mprotect(address, length, prot) == -1) {
+
         perror("mprotect");
+#ifdef LISP_FEATURE_DARWIN_JIT
+        lose("%p %lu", address, length);
+#endif
     }
 }
 
@@ -263,11 +285,12 @@ memory_fault_handler(int signal, siginfo_t *siginfo, os_context_t *context)
     FSHOW((stderr, "Memory fault at: %p, PC: %p\n", fault_addr, *os_context_pc_addr(context)));
 
 #ifdef LISP_FEATURE_SB_SAFEPOINT
-    if (!handle_safepoint_violation(context, fault_addr))
+    if (handle_safepoint_violation(context, fault_addr)) return;
 #endif
 
-    if (!gencgc_handle_wp_violation(fault_addr))
-        if(!handle_guard_page_triggered(context,fault_addr))
+    if (gencgc_handle_wp_violation(fault_addr)) return;
+
+    if (!handle_guard_page_triggered(context,fault_addr))
             lisp_memory_fault_error(context, fault_addr);
 }
 
@@ -293,19 +316,13 @@ os_install_interrupt_handlers(void)
 #endif
                                                  memory_fault_handler);
 #endif
-    }
 
-#ifdef LISP_FEATURE_SB_THREAD
-# ifdef LISP_FEATURE_SB_SAFEPOINT
-#  ifdef LISP_FEATURE_SB_THRUPTION
-    ll_install_handler(SIGURG, thruption_handler);
-#  endif
-# else
-    ll_install_handler(SIG_STOP_FOR_GC, sig_stop_for_gc_handler);
-# endif
+#ifdef LISP_FEATURE_DARWIN
+    /* Unmapped pages get this and not SIGBUS. */
+    ll_install_handler(SIGSEGV, memory_fault_handler);
 #endif
 
-    SHOW("leaving os_install_interrupt_handlers()");
+    }
 }
 
 #else /* Currently PPC/Darwin/Cheney only */
@@ -316,15 +333,15 @@ sigsegv_handler(int signal, siginfo_t *info, os_context_t *context)
     os_vm_address_t addr;
 
     addr = arch_get_bad_addr(signal, info, context);
-    if (!cheneygc_handle_wp_violation(context, addr))
-        if (!handle_guard_page_triggered(context, addr))
+    if (cheneygc_handle_wp_violation(context, addr)) return;
+
+    if (!handle_guard_page_triggered(context, addr))
             interrupt_handle_now(signal, info, context);
 }
 
 void
 os_install_interrupt_handlers(void)
 {
-    SHOW("os_install_interrupt_handlers()/bsd-os/!defined(GENCGC)");
     ll_install_handler(SIG_MEMORY_FAULT, sigsegv_handler);
 }
 
@@ -664,60 +681,3 @@ os_dlsym(void *handle, const char *symbol)
 }
 
 #endif
-
-#if defined(LISP_FEATURE_SB_WTIMER) && !defined(LISP_FEATURE_DARWIN)
-/*
- * Waitable timer implementation for the safepoint-based (SIGALRM-free)
- * timer facility using kqueue.
- */
-int
-os_create_wtimer()
-{
-    int kq = kqueue();
-    if (kq == -1)
-        lose("os_create_wtimer: kqueue");
-    return kq;
-}
-
-int
-os_wait_for_wtimer(int kq)
-{
-    struct kevent ev;
-    int n;
-    if ( (n = kevent(kq, 0, 0, &ev, 1, 0)) == -1) {
-        if (errno != EINTR)
-            lose("os_wtimer_listen failed");
-        n = 0;
-    }
-    return n != 1;
-}
-
-void
-os_close_wtimer(int kq)
-{
-    if (close(kq) == -1)
-        lose("os_close_wtimer failed");
-}
-
-void
-os_set_wtimer(int kq, int sec, int nsec)
-{
-    long long msec
-        = ((long long) sec) * 1000 + (long long) (nsec+999999) / 1000000;
-    if (msec > INT_MAX) msec = INT_MAX;
-
-    struct kevent ev;
-    EV_SET(&ev, 1, EVFILT_TIMER, EV_ADD|EV_ENABLE|EV_ONESHOT, 0, (int)msec, 0);
-    if (kevent(kq, &ev, 1, 0, 0, 0) == -1)
-        perror("os_set_wtimer: kevent");
-}
-
-void
-os_cancel_wtimer(int kq)
-{
-    struct kevent ev;
-    EV_SET(&ev, 1, EVFILT_TIMER, EV_DISABLE, 0, 0, 0);
-    if (kevent(kq, &ev, 1, 0, 0, 0) == -1 && errno != ENOENT)
-        perror("os_cancel_wtimer: kevent");
-}
-#endif
