@@ -1386,7 +1386,9 @@ veh(EXCEPTION_POINTERS *ep)
         (rip >= DYNAMIC_SPACE_START &&
          rip <= DYNAMIC_SPACE_START+dynamic_space_size);
 
-    if (code == EXCEPTION_ACCESS_VIOLATION || from_lisp)
+    if (code == EXCEPTION_ACCESS_VIOLATION ||
+        code == STATUS_HEAP_CORRUPTION ||
+        from_lisp)
         disp = handle_exception_ex(ep->ExceptionRecord, 0, ep->ContextRecord,
                                    // continue search on unhandled memory faults
                                    // if not in Lisp code. This gives foreign
@@ -1478,11 +1480,11 @@ char *dirname(char *path)
 
 // 0 - not a socket or other error, 1 - has input, 2 - has no input
 int
-socket_input_available(HANDLE socket)
+socket_input_available(HANDLE socket, long time, long utime)
 {
     int count = 0;
     int wsaErrno = GetLastError();
-    TIMEVAL timeout = {0, 0};
+    TIMEVAL timeout = {time, utime};
     fd_set readfds, errfds;
 
     FD_ZERO(&readfds);
