@@ -16,7 +16,6 @@
 #include "globals.h"
 #include "validate.h"
 #include "os.h"
-#include "sbcl.h"
 #include "arch.h"
 #include "lispregs.h"
 #include "signal.h"
@@ -28,7 +27,6 @@
 #include "getallocptr.h"
 #include "unaligned.h"
 #include "search.h"
-#include "globals.h" // for asm_routines_start,end
 
 #include "genesis/static-symbols.h"
 #include "genesis/symbol.h"
@@ -40,7 +38,6 @@
 #define UD2_INST 0x0b0f
 #define BREAKPOINT_WIDTH 1
 
-unsigned int cpuid_fn1_ecx;
 int avx_supported = 0, avx2_supported = 0;
 
 static void cpuid(unsigned info, unsigned subinfo,
@@ -72,11 +69,27 @@ static void xgetbv(unsigned *eax, unsigned *edx)
 
 #define VECTOR_FILL_T "VECTOR-FILL/T"
 
+void set_alloc_tramp_vectors(int ymm_enable)
+{
+    struct code* code = (struct code*)asm_routines_start;
+    lispobj* instructions = (lispobj*)code + code_header_words(code);
+    int index;
+    get_asm_routine_by_name("FPR-SAVE", &index);
+    if (index)
+        instructions[index] =
+            (lispobj)get_asm_routine_by_name(ymm_enable ? "SAVE-YMM" : "SAVE-XMM", 0);
+    get_asm_routine_by_name("FPR-RESTORE", &index);
+    if (index)
+        instructions[index] =
+            (lispobj)get_asm_routine_by_name(ymm_enable ? "RESTORE-YMM" : "RESTORE-XMM", 0);
+}
+
 // Poke in a byte that changes an opcode to enable faster vector fill.
 // Using fixed offsets and bytes is no worse than what we do elsewhere.
 void tune_asm_routines_for_microarch(void)
 {
     unsigned int eax, ebx, ecx, edx;
+    unsigned int cpuid_fn1_ecx = 0;
 
     cpuid(0, 0, &eax, &ebx, &ecx, &edx);
     if (eax >= 1) { // see if we can execute basic id function 1
@@ -94,7 +107,14 @@ void tune_asm_routines_for_microarch(void)
             }
         }
     }
-    SetSymbolValue(CPUID_FN1_ECX, (lispobj)make_fixnum(cpuid_fn1_ecx), 0);
+    set_alloc_tramp_vectors(avx_supported);
+    int our_cpu_feature_bits = 0;
+    // avx2_supported gets copied into bit 1 of *CPU-FEATURE-BITS*
+    if (avx2_supported) our_cpu_feature_bits |= 1;
+    // POPCNT = ECX bit 23, which gets copied into bit 2 in *CPU-FEATURE-BITS*
+    if (cpuid_fn1_ecx & (1<<23)) our_cpu_feature_bits |= 2;
+    SetSymbolValue(CPU_FEATURE_BITS, make_fixnum(our_cpu_feature_bits), 0);
+
     // I don't know if this works on Windows
 #ifndef _MSC_VER
     cpuid(0, 0, &eax, &ebx, &ecx, &edx);
@@ -113,6 +133,8 @@ void tune_asm_routines_for_microarch(void)
 void untune_asm_routines_for_microarch(void)
 {
     asm_routine_poke(VECTOR_FILL_T, 0x12, 0xEB); // Change JL to JMP
+    SetSymbolValue(CPU_FEATURE_BITS, 0, 0);
+    set_alloc_tramp_vectors(0);
 }
 
 #ifndef _WIN64
@@ -219,20 +241,20 @@ arch_internal_error_arguments(os_context_t *context)
 boolean
 arch_pseudo_atomic_atomic(os_context_t __attribute__((unused)) *context)
 {
-    return get_pseudo_atomic_atomic(arch_os_get_current_thread());
+    return get_pseudo_atomic_atomic(get_sb_vm_thread());
 }
 
 void
 arch_set_pseudo_atomic_interrupted(os_context_t __attribute__((unused)) *context)
 {
-    struct thread *thread = arch_os_get_current_thread();
+    struct thread *thread = get_sb_vm_thread();
     set_pseudo_atomic_interrupted(thread);
 }
 
 void
 arch_clear_pseudo_atomic_interrupted(os_context_t __attribute__((unused)) *context)
 {
-    struct thread *thread = arch_os_get_current_thread();
+    struct thread *thread = get_sb_vm_thread();
     clear_pseudo_atomic_interrupted(thread);
 }
 
@@ -361,7 +383,7 @@ sigtrap_handler(int __attribute__((unused)) signal,
 
     /* This is just for info in case the monitor wants to print an
      * approximation. */
-    access_control_stack_pointer(arch_os_get_current_thread()) =
+    access_control_stack_pointer(get_sb_vm_thread()) =
         (lispobj *)*os_context_sp_addr(context);
 
     /* On entry %rip points just after the INT3 byte and aims at the
@@ -406,6 +428,10 @@ sigill_handler(int __attribute__((unused)) signal,
 #endif
 
     fake_foreign_function_call(context);
+#ifdef LISP_FEATURE_LINUX
+    extern void sb_dump_mcontext(char*,void*);
+    sb_dump_mcontext("SIGILL received", context);
+#endif
     lose("Unhandled SIGILL at %p.", pc);
 }
 
@@ -604,8 +630,12 @@ lispobj fdefn_callee_lispobj(struct fdefn* fdefn) {
 extern unsigned int alloc_profile_n_counters;
 extern unsigned int max_alloc_point_counters;
 #ifdef LISP_FEATURE_SB_THREAD
+#ifdef LISP_FEATURE_WIN32
+extern CRITICAL_SECTION alloc_profiler_lock;
+#else
 extern pthread_mutex_t alloc_profiler_lock;
 #endif
+#endif
 
 static unsigned int claim_index(int qty)
 {
@@ -621,7 +651,8 @@ static unsigned int claim_index(int qty)
     return 0; // use the overflow bin(s)
 }
 
-static boolean instrumentp(uword_t* sp, uword_t** pc, uword_t* old_word)
+static boolean NO_SANITIZE_MEMORY
+instrumentp(uword_t* sp, uword_t** pc, uword_t* old_word)
 {
     int __attribute__((unused)) ret = thread_mutex_lock(&alloc_profiler_lock);
     gc_assert(ret == 0);
